@@ -10,7 +10,7 @@
 #' a character vector of host names, a positive numeric scalar,
 #' or a function.
 #' If a character vector or a numeric scalar, a `cluster` object
-#' is created using \code{\link{makeClusterPSOCK}(workers)}.
+#' is created using \code{\link[parallelly:makeClusterPSOCK]{makeClusterPSOCK}(workers)}.
 #' If a function, it is called without arguments _when the future
 #' is created_ and its value is used to configure the workers.
 #' The function should return any of the above types.
@@ -18,9 +18,8 @@
 #' @return
 #' `ClusterFuture()` returns an object of class `ClusterFuture`.
 #'
-#' @seealso
-#' To evaluate an expression using "cluster future", see function
-#' [cluster()].
+#' @section Usage:
+#' To use 'cluster' futures, use `plan(cluster, ...)`, cf. [cluster].
 #'
 #' @aliases MultisessionFuture MultisessionFuture-class
 #' @export
@@ -28,28 +27,19 @@
 #' @importFrom digest digest
 #' @name ClusterFuture-class
 #' @keywords internal
-ClusterFuture <- function(expr = NULL, substitute = TRUE, envir = parent.frame(), globals = TRUE, packages = NULL, local = !persistent, persistent = FALSE, workers = NULL, ...) {
+ClusterFuture <- function(expr = NULL, substitute = TRUE, envir = parent.frame(), persistent = FALSE, workers = NULL, ...) {
   if (substitute) expr <- substitute(expr)
   
   stop_if_not(is.logical(persistent), length(persistent) == 1L,
               !is.na(persistent))
 
-  ## WORKAROUND: Skip scanning of globals if already done /HB 2021-01-18
-  if (!isTRUE(attr(globals, "already-done", exact = TRUE))) {
-    gp <- getGlobalsAndPackages(expr, envir = envir, persistent = persistent, globals = globals)
-    globals <- gp$globals
-    packages <- c(packages, gp$packages)
-    expr <- gp$expr
-    gp <- NULL
-  }
-
   args <- list(...)
 
   ## Which '...' arguments should be passed to Future() and 
   ## which should be passed to makeClusterPSOCK()?
   future_args <- !is.element(names(args), makeClusterPSOCK_args())
   
-  future <- do.call(MultiprocessFuture, args = c(list(expr = quote(expr), substitute = FALSE, envir = envir, globals = globals, packages = packages, local = local, node = NA_integer_, persistent = persistent), args[future_args]), quote = FALSE)
+  future <- do.call(MultiprocessFuture, args = c(list(expr = quote(expr), substitute = FALSE, envir = envir, persistent = persistent, node = NA_integer_), args[future_args]), quote = FALSE)
 
   future <- do.call(as_ClusterFuture, args = c(list(future, workers = workers), args[!future_args]), quote = TRUE)
 
@@ -114,7 +104,7 @@ run.ClusterFuture <- function(future, ...) {
 
   workers <- future$workers
   expr <- getExpression(future)
-  persistent <- future$persistent
+  persistent <- isTRUE(future$persistent)
 
   ## FutureRegistry to use
   reg <- sprintf("workers-%s", attr(workers, "name", exact = TRUE))
@@ -370,7 +360,7 @@ receiveMessageFromWorker <- function(future, ...) {
   ack <- tryCatch({
     msg <- recvResult(node)
     TRUE
-  }, simpleError = function(ex) ex)
+  }, error = function(ex) ex)
 
   if (inherits(ack, "error")) {
     if (debug) mdebugf("- parallel:::recvResult() produced an error: %s", conditionMessage(ack))
@@ -491,7 +481,7 @@ receiveMessageFromWorker <- function(future, ...) {
 }
 
 
-requestNode <- function(await, workers, timeout = getOption("future.wait.timeout", 30 * 24 * 60 * 60), delta = getOption("future.wait.interval", 0.2), alpha = getOption("future.wait.alpha", 1.01)) {
+requestNode <- function(await, workers, timeout = getOption("future.wait.timeout", 30 * 24 * 60 * 60), delta = getOption("future.wait.interval", 0.01), alpha = getOption("future.wait.alpha", 1.01)) {
   debug <- getOption("future.debug", FALSE)
   
   stop_if_not(inherits(workers, "cluster"))
@@ -564,7 +554,7 @@ requestNode <- function(await, workers, timeout = getOption("future.wait.timeout
 #' @export
 getExpression.ClusterFuture <- local({
   tmpl_expr_conditions <- bquote_compile({
-    ...future.sendCondition <- local({
+    ...future.makeSendCondition <- local({
       sendCondition <- NULL
 
       function(frame = 1L) {
@@ -602,7 +592,7 @@ getExpression.ClusterFuture <- local({
     withCallingHandlers({
       .(expr)
     }, immediateCondition = function(cond) {
-      sendCondition <- ...future.sendCondition()
+      sendCondition <- ...future.makeSendCondition()
       sendCondition(cond)
 
       ## Avoid condition from being signaled more than once
@@ -723,6 +713,7 @@ post_mortem_cluster_failure <- function(ex, when, node, future) {
   ## (a) Did a localhost worker process terminate?
   if (!is.null(host)) {
     if (localhost && is.numeric(pid)) {
+      pid_exists <- import_parallelly("pid_exists")
       alive <- pid_exists(pid)
       if (is.na(alive)) {
         msg2 <- "Failed to determined whether a process with this PID exists or not, i.e. cannot infer whether localhost worker is alive or not"
