@@ -2,10 +2,8 @@
 #'
 #' @usage fassignment \%stdout\% capture
 #'
-#' @inheritParams multiprocess
-#' 
 #' @param fassignment The future assignment, e.g.
-#'        \code{x \%<-\% \{ expr \}}.
+#'        `x %<-% { expr }`.
 #'
 #' @param capture If TRUE, the standard output will be captured, otherwise not.
 #'
@@ -18,6 +16,7 @@
   args <- getOption("future.disposable", list())
   args["stdout"] <- list(capture)
   options(future.disposable = args)
+  on.exit(options(future.disposable = NULL))
 
   eval(fassignment, envir = envir, enclos = baseenv())
 }
