@@ -36,47 +36,55 @@
 #' system environment variable \env{R_FUTURE_PLAN}.
 #' To reset the strategy back to the default, use `plan("default")`.
 #'
-#' @section Implemented evaluation strategies:
+#' @section Built-in evaluation strategies:
+#' The \pkg{future} package provides the following built-in backends:
+#'
 #' \itemize{
 #'  \item{[`sequential`]:}{
-#'    Resolves futures sequentially in the current \R process.
-#'  }
-#'  \item{[`transparent`]:}{
-#'    Resolves futures sequentially in the current \R process and
-#'    assignments will be done to the calling environment.
-#'    Early stopping is enabled by default.
+#'    Resolves futures sequentially in the current \R process, e.g.
+#'    `plan(sequential)`.
 #'  }
 #'  \item{[`multisession`]:}{
 #'    Resolves futures asynchronously (in parallel) in separate
-#'    \R sessions running in the background on the same machine.
+#'    \R sessions running in the background on the same machine, e.g.
+#'    `plan(multisession)` and `plan(multisession, workers = 2)`.
 #'  }
 #'  \item{[`multicore`]:}{
 #'    Resolves futures asynchronously (in parallel) in separate
 #'    _forked_ \R processes running in the background on
-#'    the same machine.  Not supported on Windows.
-#'  }
-#'  \item{[`multiprocess`]:}{(DEPRECATED)
-#'    If multicore evaluation is supported, that will be used,
-#'    otherwise multisession evaluation will be used.
+#'    the same machine, e.g.
+#'    `plan(multicore)` and `plan(multicore, workers = 2)`.
+#'    This backend is not supported on Windows.
 #'  }
 #'  \item{[`cluster`]:}{
 #'    Resolves futures asynchronously (in parallel) in separate
-#'    \R sessions running typically on one or more machines.
+#'    \R sessions running typically on one or more machines, e.g.
+#'    `plan(cluster)`, `plan(cluster, workers = 2)`, and
+#'    `plan(cluster, workers = c("n1", "n1", "n2", "server.remote.org"))`.
 #'  }
-#'  \item{[`remote`]:}{
-#'    Resolves futures asynchronously in a separate \R session
-#'    running on a separate machine, typically on a different
-#'    network.
+#' }
+#'
+#' Other package provide additional evaluation strategies.
+#' For example, the \pkg{future.callr} package implements an alternative
+#' to the `multisession` backend on top of the \pkg{callr} package, e.g.
+#' `plan(future.callr::callr, workers = 2)`.
+#' Another example is the \pkg{future.batchtools} package, which implements,
+#' on top of the \pkg{batchtools} package, e.g.
+#' `plan(future.batchtools::batchtools_slurm)`.
+#' These types of futures are resolved via job schedulers, which typically
+#' are available on high-performance compute (HPC) clusters, e.g. LSF,
+#' Slurm, TORQUE/PBS, Sun Grid Engine, and OpenLava.
+#'
+#' The following future strategies are _deprecated_ and must not be used:
+#'
+#' \itemize{
+#'  \item{[`multiprocess`]:}{(DEPRECATED since future 1.20.0)
+#'    If multicore evaluation is supported, that will be used,
+#'    otherwise multisession evaluation will be used.
+#'    _Please use `multisession`, or possibly `multicore` instead._
 #'  }
 #' }
 #' 
-#' Other package may provide additional evaluation strategies.
-#' Notably, the \pkg{future.batchtools} package implements a
-#' type of futures that will be resolved via job schedulers
-#' that are typically available on high-performance compute
-#' (HPC) clusters, e.g. LSF, Slurm, TORQUE/PBS, Sun Grid Engine,
-#' and OpenLava.
-#'
 #' To "close" any background workers (e.g. `multisession`), change
 #' the plan to something different; `plan(sequential)` is recommended
 #' for this.
@@ -133,26 +141,74 @@ plan <- local({
   ## Stack of type of futures to use
   stack <- defaultStack
 
-  warn_about_multiprocess <- local({
+  assert_no_disallowed_strategies <- function(stack) {
+    noplans <- getOption("future.plan.disallow")
+    if (length(noplans) == 0L) return()
+
+    for (kk in seq_along(stack)) {
+      evaluator <- stack[[kk]]
+      if (!inherits(evaluator, noplans)) next
+      clazz <- class(evaluator)[1]
+      if (!clazz %in% noplans) next  ## <== sic!
+
+      stop(FutureError(sprintf("Can not use %s in the future plan because it is on the list of future strategies that are not allow per option 'future.plan.disallow': %s", sQuote(clazz), paste(sQuote(noplans), collapse = ", "))))
+    }
+  }
+
+  evaluator_uses <- function(evaluator, strategy) {
+    if (!inherits(evaluator, strategy)) return(FALSE)
+    ## NOTE: Yes, we are indeed inspecting the 'class' attribute itself
+    class <- class(evaluator)
+    if (class[1] == strategy) return(TRUE)
+    if (length(class) == 1L) return(FALSE)
+    if (class[1] == "tweaked" && class[2] == strategy) return(TRUE)
+    FALSE
+  }
+
+  warn_about_deprecated <- function(stack, strategy, fmtstr, ignore = NULL, defunct = NULL) {
+    for (kk in seq_along(stack)) {
+      if (evaluator_uses(stack[[kk]], strategy)) {
+        if (is.null(ignore)) ignore <- getOption("future.deprecated.ignore")
+        if (!is.element(strategy, ignore)) {
+          if (is.null(defunct)) defunct <- getOption("future.deprecated.defunct")
+          if (is.element(strategy, defunct)) {
+            msg <- sprintf(fmtstr, strategy, "defunct")
+            dfcn <- .Defunct
+          } else {
+            msg <- sprintf(fmtstr, strategy, "deprecated")
+            dfcn <- .Deprecated
+          }
+          dfcn(msg = msg, package = .packageName)
+        }
+      }
+    }
+  }
+
+  warn_about_multiprocess <- function(stack) {
+    warn_about_deprecated(stack, strategy = "multiprocess", fmtstr = sprintf("Strategy '%%s' is %%s in future (>= 1.20.0) [2020-10-30]. Instead, explicitly specify either 'multisession' (recommended) or 'multicore'. Starting with future 1.31.0 [2023-01-31], 'multiprocess' is the same as 'sequential'."))
+  }
+
+  warn_about_remote <- function(stack) {
+    warn_about_deprecated(stack, strategy = "remote", fmtstr = "Strategy '%s' is %s in future (>= 1.30.0) [2022-12-15]. Instead, use plan(cluster, ..., persistent = TRUE).", ignore = "", defunct = "remote")
+  }
+
+  warn_about_transparent <- function(stack) {
+    warn_about_deprecated(stack, strategy = "transparent", fmtstr = "Strategy '%s' is %s in future (>= 1.28.0) [2022-09-02]. It was designed to simplify interactive troubleshooting, but is now superseded by plan(sequential, split = TRUE).", defunct = "transparent")
+  }
+
+  warn_about_multicore <- local({
     .warn <- TRUE
 
     function(stack) {
       if (!.warn) return()
 
-      ## Is deprecated 'multiprocess' used?    
+      ## Is 'multicore' used despite not being supported on the current
+      ## platform?    
       for (kk in seq_along(stack)) {
-        evaluator <- stack[[kk]]
-        if (inherits(evaluator, "multiprocess") && 
-            class(evaluator)[1] == "multiprocess") {  ## <== sic!
-          ignore <- Sys.getenv("R_FUTURE_DEPRECATED_IGNORE", "")
-          ignore <- getOption("future.deprecated.ignore", ignore)
-          ignore <- unlist(strsplit(ignore, split="[, ]", fixed = FALSE))
-          if (!is.element("multiprocess", ignore)) {
-            ## Warn only once
-            .warn <<- FALSE
-            .Deprecated(msg = sprintf("Strategy 'multiprocess' is deprecated in future (>= 1.20.0). Instead, explicitly specify either 'multisession' or 'multicore'. In the current R session, 'multiprocess' equals '%s'.", if (supportsMulticore()) "multicore" else "multisession"), package = .packageName)
-          }
-
+        if (evaluator_uses(stack[[kk]], "multicore")) {
+          supportsMulticore(warn = TRUE)
+          ## Warn only once, if at all
+          .warn <<- FALSE
           break
         }
       }
@@ -165,6 +221,7 @@ plan <- local({
 
   plan_init <- function() {
     evaluator <- stack[[1L]]
+
     init <- attr(evaluator, "init", exact = TRUE)
     if (identical(init, TRUE)) {
       debug <- getOption("future.debug", FALSE)
@@ -195,7 +252,12 @@ plan <- local({
       }
 
       if (!identical(res, NA)) {
-        stop(FutureError("Initialization of plan() failed, because the value of the test future is not the expected one: ", sQuote(res)))
+        res <- if (is.null(res)) {
+          "NULL"
+        } else {
+          paste(sQuote(res), collapse = ", ")
+        }
+        stop(FutureError(sprintf("Initialization of plan() failed, because the value of the test future is not NA as expected: %s", res)))
       }
       
       if (debug) {
@@ -238,7 +300,12 @@ plan <- local({
       mprint(newStack)
     }
 
+    assert_no_disallowed_strategies(newStack)
+
     warn_about_multiprocess(newStack)
+    warn_about_remote(newStack)
+    warn_about_transparent(newStack)
+    warn_about_multicore(newStack)
 
     stack <<- newStack
 
@@ -251,7 +318,7 @@ plan <- local({
     ## Sanity checks
     nbrOfWorkers <- nbrOfWorkers()
     if (getOption("future.debug", FALSE)) {
-      mdebugf(sprintf("plan(): nbrOfWorkers() = %g", nbrOfWorkers))
+      mdebugf(sprintf("plan(): nbrOfWorkers() = %.0f", nbrOfWorkers))
     }
     stop_if_not(is.numeric(nbrOfWorkers), length(nbrOfWorkers) == 1L, 
                 !is.na(nbrOfWorkers), nbrOfWorkers >= 1L)
@@ -305,6 +372,11 @@ plan <- local({
 
     ## Set new stack?
     if (is.function(strategy)) {
+      ## Tweak the strategy function?
+      if (length(targs) > 0) {
+        args <- c(list(strategy), targs, penvir = parent.frame())
+        strategy <- do.call(tweak, args = args)
+      }
       strategy <- list(strategy)
     }
 
@@ -376,16 +448,21 @@ plan <- local({
         } else {
           isSymbol <- sapply(strategyT, FUN = is.symbol)
           if (!all(isSymbol)) {
-            targs <- c(targs, strategyT[-1L])
-            strategy <- strategyT[[1L]]
+            strategy <- eval(strategyT[[1L]], envir = parent.frame(), enclos = baseenv())
+            if (length(strategyT) > 1L) {
+              ## Tweak this part of the future strategy
+              args <- c(list(strategy), strategyT[-1L], penvir = parent.frame())
+              strategy <- do.call(tweak, args = args)
+            }
+          } else {
+            strategy <- eval(strategy, envir = parent.frame(), enclos = baseenv())
           }
-          strategy <- eval(strategy, envir = parent.frame(), enclos = baseenv())
         }
       }
 
       ## Tweak future strategy accordingly
       args <- c(list(strategy), targs, penvir = parent.frame())
-      tstrategy <- do.call(tweak, args = args)
+      tstrategy <- do.call(tweak, args = args, quote = TRUE)
 
       ## Setup a new stack of future strategies (with a single one)
       newStack <- list(tstrategy)
@@ -417,8 +494,7 @@ plan <- local({
 
 
 supportedStrategies <- function(strategies = c("sequential", "multicore",
-                                               "multisession", "multiprocess",
-                                               "cluster")) {
+                                               "multisession", "cluster")) {
   if (!supportsMulticore()) strategies <- setdiff(strategies, "multicore")
   strategies
 }
